function IntegrationIndexCorrection = OutOfDetectorCorrection(IntegrationIndexCounting,RadiusEdges,RadiusUnit,Azi,Sample2DetectorDistance,DetectorSize,UseGPU)
    
    % COMPENSATION FOR DEBYE-SCHERRER RINGS THAT ARE NOT FULL. SHOULD ONLY
    % BE USED IF THERE ARE RINGS THAT ARE NOT FULL.
    
    % Integrate 1s to count number of pixels going into each bin, and
    % compare this to the theoretical number of pixels that should be in 
    % each bin. The higher the "scaling" the more precise this correction
    % will be.
    
    PixCountInBin_actual = MatFRAIA.integrate(IntegrationIndexCounting,ones(prod(DetectorSize),1));
    
    % With the bin edges of the radial bins, in units of pixels, the
    % theoretical number of pixels in this bin is simply the definite
    % integral (LaTeX style):
    %
    % Number of pixels (from r_a to r_b) = $\int_{r_a}^{r_b} pi*r^2 dr$
    %                                    = $[1/3*pi*r^3]_{r_a}^{r_b}$
    %                                    = $ 1/3*pi*(r_b^3-r_a^3) $
    %
    % And if there's more than one azimuthal bin/segment:
    % 
    % $ = 1/3*pi*(r_b^3-r_a^3)/(\Chi_b-\Chi_a) $, where $\Chi$ is simply
    % the start and beginning of the azimuthal bin (the edges).
    
    switch RadiusUnit
        case 'degrees' % input will be in radians
            RadiusBinEdgesInPixels = tan(RadiusEdges).*Sample2DetectorDistance;
        case 'pixels' % input will be in pixels
            RadiusBinEdgesInPixels = RadiusEdges;
    end
            
    r_a = RadiusBinEdgesInPixels(1:end-1).';
    r_b = RadiusBinEdgesInPixels(2:end).';
    
    Chi_b_minus_a = (Azi.max-Azi.min)/Azi.N; 
    PixCountInBin_theory = pi*(r_b.^2-r_a.^2)./(2*pi./Chi_b_minus_a);
    Corr = PixCountInBin_theory./PixCountInBin_actual;
    
    
    [i,j,v] = find(IntegrationIndexCounting.TRANSMAT);
    % Remove all but the first ADJUSTMENTS(i) that is nan (for
    % unique i's), i.e. i = [1,1,1,1,2,3,4,5,5,5], will become: 
    % i = [1, 2,3,4,5,5,5] if ADJUSTMENT(1) is nan.
    % Keep only the first bin index (i), if it points to a bin that is
    % ultimately nan'ed, since no pixels (j) that points into a bin 
    % that is eventually nan'ed will be a waste of computation time.

    [~,ia] = unique(i,'first'); 
    firstNan = isnan(Corr(i(ia)));

    shouldBeNan = ia(firstNan);
    allNans =  isnan(Corr(i));
    allNans(shouldBeNan) = false;  
    i(allNans) = [];
    v(allNans) = [];
    j(allNans) = [];
    % ADJUSTMENT is applied and remove all the contribution from masked
    % out pixels:
    v = Corr(i);%.*newmask(j);
    IntegrationIndexCorrection = IntegrationIndexCounting;
    IntegrationIndexCorrection.TRANSMAT = MatFRAIA.subfunctions.sparseInSteps(i,j,v,size(IntegrationIndexCounting.TRANSMAT),UseGPU);