function [CONVERTED, READ_FROM_PONI] = readAndOrConvert(PONI_FILE_PATH,varargin)
	% Needs the following (example):
	% 	Poni1= 0.09144741204531871; % in meters
	% 	Poni2= 0.08691428217726348; % in meters
	% 	Rot1 = 0.004861078016557119; % in rad
	% 	Rot2 = -0.00039455468604525417; % in rad
	% 	Not needed: Rot3= 1.8208975092471103e-07; % in rad
	% 	pixsize = 75e-6; % in meters
	% 	Distance = 122*e-3; % in meters
	% 	%
	if isempty(PONI_FILE_PATH)
		PONI_FILE_CONTENT = struct;
	else
		PONI_FILE_CONTENT = MatFRAIA.subfunctions.PONI.read(PONI_FILE_PATH);
	end
	INP = inputParser();
	INP.StructExpand = true;
	INP.KeepUnmatched = true;
	INP = addParameterIfPresent(INP, 'Distance',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Poni1',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Poni2',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Rot1',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Rot2',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Rot3',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'Wavelength',	PONI_FILE_CONTENT);
	INP = addParameterIfPresent(INP, 'pixsize',	PONI_FILE_CONTENT);
	INP.parse(varargin{:})
	INPres = INP.Results;
	
	% Check if some inputs are missing both from .poni file, as well as
	% from name-value pair input
	WhatIsMissing(INPres);
	
	ROT	= [INPres.Rot1, INPres.Rot2, INPres.Rot3];
	dist	= INPres.Distance;
	pixsize = repmat(INPres.pixsize, 1, 2/numel(INPres.pixsize));
	PONI	= [INPres.Poni1, INPres.Poni2];
	
	cos_tilt = cos(ROT(1)).*cos(ROT(2));
	sin_tilt = sqrt(1.0 - cos_tilt .* cos_tilt);
	tan_tilt = sin_tilt ./ cos_tilt;
	degrees =@(x) x./pi*180;
	
	% Rewritten to MATLAB code from pyFAI class GEOMETRY (in order to 
	% ensure correct conversion from .poni files to FIT2D geometry):
	
	% This is tilt plane rotation
	if sin_tilt == 0
		% tilt plan rotation is undefined when there is no tilt!, does not matter
		cos_tilt = 1.0;
		%sin_tilt = 0.0;
		cos_tpr = 1.0;
		sin_tpr = 0.0;
	else
		cos_tpr = max(-1, min(1, -cos(ROT(2)) .* sin(ROT(1)) ./ sin_tilt));
		sin_tpr = sin(ROT(2)) ./ sin_tilt;
	end
	directDist = 1.0e3 * dist ./ cos_tilt;
	tilt = degrees(acos(cos_tilt));
	if sin_tpr < 0
		tpr = -degrees(acos(cos_tpr));
	else
		tpr = degrees(acos(cos_tpr));
	end

	centerX = (PONI(2) + dist .* tan_tilt .* cos_tpr) ./ pixsize(2);
	if abs(tilt) < 1e-5 % in degree
		centerY = (PONI(1)) ./ pixsize(1);
	else
		centerY = (PONI(1) + dist .* tan_tilt .* sin_tpr) ./ pixsize(1);
	end
	% END OF pyFAI conversion!
	
	CONVERTED.detector.distance = directDist;
	CONVERTED.detector.pixsize = pixsize*1e3;
	CONVERTED.center.x = centerX;
	CONVERTED.center.y = centerY;
	CONVERTED.tilt.delta = tilt;
	CONVERTED.tilt.rot = tpr;
	if isfield(PONI_FILE_CONTENT,'RAW')
		INPres.RAW = PONI_FILE_CONTENT.RAW;
	end
	READ_FROM_PONI = INPres;

end

function INP = addParameterIfPresent(INP, Param, PONI_FILE_CONTENT)
	if isfield(PONI_FILE_CONTENT, Param)
		INP.addParameter( Param,	PONI_FILE_CONTENT.(Param))
	else
		INP.addParameter(Param,	[])
	end
end

function WhatIsMissing(INP)
	theFields = fields(INP);
	emptyFields = cellfun(@(X) isempty(INP.(X)),theFields);
	if any(emptyFields)
		missingStr = '';
		emptyIdxs = find(emptyFields);
		if numel(emptyIdxs) == 1 && strcmp(theFields{emptyIdxs},'Wavelength')
			return
		end
		for emptyIdx = emptyIdxs(1:end-1)
			missingStr = [missingStr, theFields{emptyIdx}, ', '];
		end
		missingStr = [missingStr, theFields{emptyIdxs(end)}];
		
		[~,NAME] = fileparts(mfilename('fullpath'));
		fullNAME = [NAME,'.m'];
		
		st = dbstack;
		
		if any(strcmp('indexing.m', {st(:).file}))
			error('\n  Missing needed input from .poni-file and it was not given as name-value pair: \n\n\t\t%s.\n\n   Please add these to the .poni-file or as a field in the EXPERIMENT_INFO-struct (or the EXPERIMENT_INFO.PONI-struct) to the MatFRAIA.indexing-function like:\n    %s',missingStr,['EXPERIMENT_INFO.PONI.',theFields{emptyIdxs(end)},' = <INSERT VALUE>; [IntIdx,info] = MatFRAIA.indexing(mask,INTEGRATION_OPTIONS,EXPERIMENT_INFO);']);
% 		else
% 			error('\n  Missing needed input from .poni-file and it was not given as name-value pair: \n\n\t\t%s.\n\n   Please add these to the .poni-file or as name-value pair input to the %s-function',NAME);
		elseif strcmp(fullNAME,st(1).file)
% 			st =
% 			namestr = st.name
			error('\n  Missing needed input from .poni-file and it was not given as name-value pair: \n\n\t\t%s.\n\n   Please add these to the .poni-file (easiest) or as name-value pair input to the PONI_readAndConvert-function like:\n    %s',missingStr,['PONI_readAndConvert(PONI_FILE_PATH, ... ,''',theFields{emptyIdxs(end)},''',<Insert value>)']);
			
		end
	end
end


	