function [INTIDX, infoOutput, INTIDXCount] = indexing(mask,EXPERIMENT_INFO,INTEGRATION_OPTIONS)
% Integration/Unfolding indexing version 0.5 (Splitting of pixels and calculation of remapping from cartesian to polar coordinates)
% EXAMPLE OF USE AND VARIOUS INFORMATION:
% 	path2mydata = '/thepath/to/some/data';
% 	mask = MatFRAIA.detectorMasks.EIGER4M();
%
% 	% Struct containing integration options (which all has default values if not specified)
% 	% N.B.: not all settings are shown in this example!
% 	INTEGRATION_OPTIONS =...
% 			struct( 'Azi',   struct('N',1,   'min',0,'max',2*pi),...
% 				'Radius',struct('N',1500,'min',0,'max',35),...
% 				'Scaling',                      5,...
% 				'MaskCorrectionThreshold',      0.4,...
% 				'doLorentzCorrection',          true,...
% 				'UseGPU',                       false,...
% 				'doMaskCorrection',             true,...
% 				'ShiftAzi',                     -pi/2,...
% 				'doOutOfDetectorCorrection',    true);
%
% 	% Struct containing needed info about experimental setup:
% 	EXPERIMENT_INFO.detector.distance = 145.1693; % [mm], Sample-to-detector distance
% 	EXPERIMENT_INFO.detector.pixsize = 0.0750; % [mm], Pixel size on detector
% 	EXPERIMENT_INFO.center.y = 1105.86214; % [pixels], Vertical-position of beam center
% 	EXPERIMENT_INFO.center.x = 1205.66809; % [pixels], Horizontal-position of beam center
% 	EXPERIMENT_INFO.tilt.delta = 0; % [degrees]
% 	EXPERIMENT_INFO.tilt.rot = 0; % [degrees]
%
% 	%% Indexing:
% 	[INTIDX, INFO] = MatFRAIA.indexing(mask,EXPERIMENT_INFO,INTEGRATION_OPTIONS);
%
% 	%% Data loading:
% 	% a for/while loop could be placed here, for continuous loading and integration of data
% 	h5hierarchicalPath = '/entry/data/data'; % path inside the h5 file leading to the data.
% 	path2h5file = fullfile( path2mydata, 'someh5filename.h5' );
% 	h5data = h5read( path2h5file, h5archievepath );
%
% 	%% Integration
% 	if ~isa(h5data,'double')
% 		% Casting to double is needed because the matrix multiplication
% 		% with a sparse matrix is allowed only with 'double' as of February 2020.
% 		h5data = double(h5data);
% 	end
% 	INT = MatFRAIA.integrate(INTIDX, reshape(h5data,[],size(h5data,3)) );
%
%

%% !!!!! CHANGELOG !!!!!
% --- UPDATE mid August 2020
%  -- FLAT (LINEAR) PIXEL INTEGRATION
%   - Made it possible to now omit everything in the Radius struct, as this
%	  is now calculated automatically to cover the entire "detector". This
%	  is used when RadiusUnit and ConstantSpacingIn are set to 'pixels' and
%	  either Radius.N, Radius.max or Radius.min is omitted. When using this
%	  mode, the sample-to-detector distance (EXPERIMENT_INFO.detector.distance) 
%	  and detector pixel size (EXPERIMENT_INFO.detector.pixsize) are not 
%	  needed since they are not used.
%
% --- UPDATE 27th of January 2020
%  -- INPUT FORMAT UPDATE:
%   - Input was changed to 3 inputs:
%     1) MASK containing the mask
%     2) EXPERIMENTAL_INFO containing information regarding the
%     experimental setup (sample-to-detector distance, wavelength,
%     pixel size of detector)
%     3) INTEGRATION_INFO
%
%
% --- UPDATE PER OCTOBER 2019:
%  -- MEMORY EFFICIENT INDEXING CREATED AND OPTIMIZED
%   -

% --- UPDATE PER 17th of January 2019:
% -- IMPLEMENTED ADJUSTMENT OF WEIGHTS BASED ON CORRECTIONS WANTED:
%   - Implemented adjustments of weights in transformation matrix
%     TRANSMAT, based on which corrections that are wanted.
%   - Added command-line feedback stating current settings of the
%     indexing, thus allowing the user to cancel if some settings are
%     not correct.
% -- INCLUDED Radius.N AND Azi.N IN UnfoldIndexing VARIABLE
%   - Inclusion of Radius.N and Azi.N in UnfoldIndexing variable now
%     makes unfIntUnfolding only take 2 inputs instead of 4.

% --- UPDATE PER 7th of January 2019:
% -- ADDED LORENTZ CORRECTION
%  - Lorentz Correction compensates for the fact that a flat 2D area
%  detector, without curvature, was used to record the diffraction signal.
%  Therefore one needs to correct for this when re-binning into 2theta
%  due to geometrical differences between this re-binned scan and a true
%  2theta scan, as you would with Debye-Scherrer geometry with a strip-detector.
% -- CHANGE x and y on center, because it was wrong before!

% --- OLD UPDATE:
% -- CHANGED OUTPUT TO TRANSFORMATION MATRIX
% - Changed output to TRANSMAT which is a transformation matrix,
%   and PolCorrOrig (apply this to all RAW images before unfolding), and
%   NANCORR (automatically applied by unfolding).
% END OF CHANGELOG

%import MatFRAIA.*

%% INPUT OPTIONS (name-value pairs):
%strChecker = @(x) isstring(x) | ischar(x) | iscellstr(x);

%validateFunctionSignaturesJSON(fullfile(regexp(mfilename('fullpath'),['.*',filesep,'\+MatFRAIA'],'match','once'),'functionSignatures.json'))
INPUTS = inputParser;
INPUTS.StructExpand = true;
INPUTS.addParameter('UseGPU',				false,		@islogical);
INPUTS.addParameter('UseLowMemoryFootprintIndexing',	true,		@islogical);
INPUTS.addParameter('SuppressTiltCheck',		false,		@islogical);
INPUTS.addParameter('doOutOfDetectorCorrection',	false,		@islogical);
INPUTS.addParameter('doSolidAngleCorrection',		false,		@islogical);
INPUTS.addParameter('doMaskCorrection',			true,		@islogical);
INPUTS.addParameter('doLorentzCorrection',		true,		@islogical);
INPUTS.addParameter('doTiltCorrection',			true,		@islogical);
INPUTS.addParameter('doPolarizationCorrection',		true,		@islogical);
INPUTS.addParameter('PrintInfo',			true,		@islogical);
INPUTS.addParameter('Debugging',			false,		@islogical);
INPUTS.addParameter('PolarizationFactors',		[0.96 2],	@(x) ismatrix(x) && (numel(x) == 2));
INPUTS.addParameter('NormalizationMethod',		'none',		@(x) strcmp(x,'none') || strcmp(x,'pixelCountInBin'));
INPUTS.addParameter('ConstantSpacingIn',		'twotheta',	@(x) strcmp(x,'twotheta') || strcmp(x,'pixels'));
INPUTS.addParameter('RadiusUnit',			'degrees',	@(x) strcmp(x,'pixels') || strcmp(x,'degrees'))
INPUTS.addParameter('ShiftAzi',				0,		@isnumeric);
INPUTS.addParameter('LorentzCorrectionStartRadius',	0.5,		@isnumeric); % Radius in units of degrees (two theta)
INPUTS.addParameter('MaskCorrectionThreshold',		0.3,		@(x)isnumeric(x) && ~isempty(x));
INPUTS.addParameter('Scaling',				7,		@isnumeric)
INPUTS.addParameter('Azi',				struct('N',1,'min',0,'max',2*pi), @isstruct)
INPUTS.addParameter('Radius',				[], @isstruct)
INPUTS.parse(INTEGRATION_OPTIONS);
INPUTVARS = INPUTS.Results;

%% VARIOUS CHECKS AND INPUTS
if INPUTVARS.Debugging
	echo(mfilename,'on')
else
	echo(mfilename,'off')
end

if strcmp(get(0,'more'),'on') && INPUTVARS.PrintInfo
	% Disable "more" if it is set to "on", as the printing of
	% information in the command window will stop execution of the
	% function until the user presses enter/space/q.
	disabledMore = true;
	more off;
else
	disabledMore = false;
end

% PONI-FILE LOADING SHOULD RATHER BE DONE EXTERNALLY AND NOT BY THE
% INDEXING FUNCTION!
% % PONI-FILE loading
% if isfield(EXPERIMENT_INFO,'PONI') && isfield(EXPERIMENT_INFO.PONI,'Path') && ~isempty(EXPERIMENT_INFO.PONI.Path)
% 	% Read .PONI file:
% 	[CONVERTED, INPUTS_USED] = MatFRAIA.subfunctions.PONI.readAndConvert(EXPERIMENT_INFO.PONI.Path, rmfield(EXPERIMENT_INFO.PONI,'Path'), struct('EXPERIMENT_INFO',EXPERIMENT_INFO));
% 	% Insert read fields into EXPERIMENT_INFO, as if one had typed them
% 	% manually! HOWEVER, inputs through EXPERIMENT_INFO takes
% 	% precedence over loaded info from .PONI
% % 	EXPERIMENT_INFO
% 	EXPERIMENT_INFO = structFcn(CONVERTED,EXPERIMENT_INFO);
% % 	EXPERIMENT_INFO
% 	EXPERIMENT_INFO.PONI.InputsRead = INPUTS_USED;
% else
% 	EXPERIMENT_INFO.manuallySet = true;
% end


for inpN = nargin:-1:1; inputnames{inpN} = inputname(inpN); end
[INPUTVARS, EXPERIMENT_INFO] = MatFRAIA.subfunctions.performChecksAndInfo(inputnames,mask,EXPERIMENT_INFO,INTEGRATION_OPTIONS,INPUTS,INPUTVARS,nargin);
% Enable "more" if it was previously disabled!
if disabledMore; more on; end

Radius	= INPUTVARS.Radius;
Azi	= INPUTVARS.Azi;
scaling	= INPUTVARS.Scaling;
% Prepare structures containing Indexing matrix called INTIDX,
% etc.:
[INTIDX, INTIDXCount, INTIDXCount_segment(scaling), INTIDX_segment(scaling)] = deal(MatFRAIA.subfunctions.prepareINTIDXstruct(Radius.N,Azi.N));
% 	reduceIndexDatatypes = false;
% 	if INPUTVARS.MATLABversionYear >= 2020
% 		% The 2020a version of MATLAB introduced the possibility of
% 		% sparse matrix generation with input index of multiple
% 		% datatypes.
% 		reduceIndexDatatypes = true;
% 		smallestDataTypeForQP = reduceToIntDataType( numel(mask) );
% 		smallestDataTypeForQB = reduceToIntDataType( Radius.N*Azi.N );
% 	elseif INPUTVARS.MATLABversionYear < 2018
% 		% The 2018a version of MATLAB introduced sum, mean, median,
% 		% and std with DIMvector/'all'-input, which is used within
% 		% subfunctions of some of the corrections.
% 		% Fix the abscence of these by calling other (looped)
% 		% functions or reshape input so that it can be done in one go!
%
% 	end

starttime = tic;
%% START OF INDEXING:
niceTime = @(t) datestr(datenum(0,0,0,0,0,toc(t)),'HH:MM:SS.FFF');
% BUILD A GRID OF PIXEL CENTERS:
idxRowsY = linspace(0+1/(2*scaling),size(mask,1)-1/(2*scaling),size(mask,1)*scaling)-EXPERIMENT_INFO.center.y;
idxColsX = linspace(0+1/(2*scaling),size(mask,2)-1/(2*scaling),size(mask,2)*scaling)-EXPERIMENT_INFO.center.x;


% Only turn idxColsX and idxRowsY into matrices of repeating values, if
% tilt-correction is performed, as this is the only case where, e.g.,
% x changes with y changing. (it is much more memory efficient to
% simply use the x and y coordinates as vectors).
if INPUTVARS.UseLowMemoryFootprintIndexing
	ChunkScaling = scaling:size(mask,2);
	ChunkSplitting = find(mod(size(mask,2)./(ChunkScaling),1)==0,1);
	ChunkSize = size(mask,2)/(ChunkScaling(ChunkSplitting));
else
	ChunkSize = size(mask,2);
end

allColIdx = repelem(1:size(mask,2),1,scaling);

IdxFcn = @(X,S) X( ChunkSize*(S-1)+1 : ChunkSize*S );
NOsegments = size(mask,2)*scaling/ChunkSize;
for SEGMENT = 1:NOsegments
	%         if INPUTVARS.Debugging; fprintf('   %s - Running segment: %d\n',niceTime(starttime),SEGMENT); end
	if INPUTVARS.doTiltCorrection
		[xscaled,yscaled] = meshgrid(IdxFcn(idxColsX,SEGMENT),idxRowsY);
	else
		[xscaled, yscaled] = deal(IdxFcn(idxColsX,SEGMENT),idxRowsY.');
	end

	if isfield(EXPERIMENT_INFO,'detector') && isfield(EXPERIMENT_INFO.detector,'distance') && ~isempty(EXPERIMENT_INFO.detector.distance) && (EXPERIMENT_INFO.detector.distance ~= 0) && ~isnan(EXPERIMENT_INFO.detector.distance)
		samp2det = EXPERIMENT_INFO.detector.distance./EXPERIMENT_INFO.detector.pixsize;
		samp2det = samp2det(1);
	else
		samp2det = 0;
	end
	% TILT CORRECTION:
	if INPUTVARS.doTiltCorrection
		% tilt.det = EXPERIMENT_INFO.detector.distance/EXPERIMENT_INFO.pix.size; (automatically calculated)
		% tilt.rot is the rotation around the normal to the detector, and
		% tilt.delta is how much the normal to the detector deviates from
		% the beam vector (or rather the anti-parallel beam vector), i.e. if
		% given vector in the detector plane is perpenducular to the
		% incident beam vector: delta = 0; thus tilt.rot should also be 0.

		[xscaled, yscaled] = MatFRAIA.corrections.Tilt(xscaled, yscaled, EXPERIMENT_INFO.tilt.rot, EXPERIMENT_INFO.tilt.delta, samp2det);
	end

	%% (re)-binning
	%                                                                            if INPUTVARS.Debugging; fprintf('   %s - Calculating Azimuthal angles to pixel centers.\n',niceTime(starttime)); end
	% Pixel Azimuthal COORDINATES (on detector) FROM CARTESIAN GRID
	AZI = mod(atan2(yscaled,xscaled)+INPUTVARS.ShiftAzi,2*pi);
	%                                                                            if INPUTVARS.Debugging; fprintf('   %s - Discretizing Azimuthal angles.\n',niceTime(starttime)); end
	% Discretize Azimuthal coordinates (to bins)
	[~,qAziEdges,qAzi] = histcounts(AZI,Azi.N,'BinLimits',[Azi.min, Azi.max]);

	if ~INPUTVARS.doPolarizationCorrection
		%                                                                            if INPUTVARS.Debugging; fprintf('   %s - Clearing AZI variable due to no use from hereon.\n',niceTime(starttime)); end
		clearvars AZI
	end
	%                                                                            if INPUTVARS.Debugging; fprintf('   %s - Calculating radial coordinate (in pixels) to each pixel center.\n',niceTime(starttime)); end
	% Pixel Radial COORDINATES (on detector) FROM CARTESIAN GRID
	R = sqrt(xscaled.^2 + yscaled.^2);

	
	% Two theta is needed for the three following cases
	if INPUTVARS.doLorentzCorrection || INPUTVARS.doPolarizationCorrection || strcmp(INPUTVARS.ConstantSpacingIn,'twotheta')
		TWOTHETA = MatFRAIA.subfunctions.radius2twotheta(R,samp2det);
	end
	switch INPUTVARS.ConstantSpacingIn
		case 'twotheta'
			switch INPUTVARS.RadiusUnit
				case 'degrees'
					theLimits = [Radius.min, Radius.max]./180*pi;
				case 'pixels'
					theLimits = MatFRAIA.subfunctions.radius2twotheta([Radius.min, Radius.max],samp2det);
				case 'q'
					error('Integration to q-space is NOT implemented yet! But will be ASAP.')
			end
			% Discretize Radial bins (in twotheta)
			[~,qREdges,qR] = histcounts(TWOTHETA,Radius.N,'BinLimits',theLimits);
			% Return center of bins.
			BINEDGES.TwoThetaInRadians = qREdges;
			BINCENTERS.TwoThetaInRadians = qREdges(1:end-1)+diff(qREdges,1,2)/2;
		case 'pixels'
			switch INPUTVARS.RadiusUnit
				case 'degrees'
					theLimits = 180/pi.*MatFRAIA.subfunctions.twotheta2radius([Radius.min, Radius.max],samp2det);
				case 'pixels'
					theLimits = [Radius.min, Radius.max];
			end
			% Discretize Radial bins (in pixels)
			[~,qREdges,qR] = histcounts(R,Radius.N,'BinLimits',theLimits);
			BINEDGES.RadiusFromCenterInPixels = qREdges;
			BINCENTERS.RadiusFromCenterInPixels = qREdges(1:end-1)+diff(qREdges,1,2)/2;

	end
	BINEDGES.AziInRadians = qAziEdges;
	BINCENTERS.AziInRadians = qAziEdges(1:end-1)+diff(qAziEdges,1,2)/2;
	if ~INPUTVARS.doLorentzCorrection && ~INPUTVARS.doPolarizationCorrection
		clearvars TWOTHETA R
	end
	if INPUTVARS.MATLABversionYear >= 2020
		qR = uint32(qR);
		qAzi = uint32(qAzi);
	end

	% Find all pixels which has not been assigned to a bin, and flag for removal
	rmIdx = qR == 0 | qAzi == 0;
	
	keepIdx = ~rmIdx;
	if Azi.N == 1
		qBins = qR(keepIdx);
	else
		qBins = qR(keepIdx) + (qAzi(keepIdx)-1)*Radius.N;
	end
	
	clearvars qR qAzi % Free up memory
	
	% qPix = Indices of original pixels that each subpixel belong to
	% (these needs to be summed into weights, one weight per
	% original pixel per bin).
	pixIdx = IdxFcn(allColIdx, SEGMENT);
	qPix = repelem((1:size(mask,1)).',scaling,1) + (pixIdx-1).*size(mask,1);
	% equivalent to 
	%qPix = reshape(1:numel(mask),size(mask)); % index in image (dimensions of original image);
	%qPix = imresize(qPix,scaling,'nearest');  % Conversion index from original to scaled Im(org2scaled) = scaledIm;
	%% INDEXING
	% 		IDX = ~isnan(qBins);
	qPix = qPix(keepIdx);
	%%%%% NEW WAY END

	if INPUTVARS.MATLABversionYear >= 2020
		qPix = uint32(qPix);
	end
	if ~(isempty(qBins) || isempty(qPix))
		VAL = ones(size(qBins));
		% 			qIndex = qBins + (qPix-1)*(Radius.N*Azi.N);
		if exist('TWOTHETA','var')
			TWOTHETA = TWOTHETA(keepIdx);
			% 				TWOTHETA = TWOTHETA(IDX);
		end
		if exist('AZI','var')
			AZI = AZI(keepIdx);
			% 				AZI = AZI(IDX);
		end
		%clearvars qPix qBins % Free up memory
		%% Corrections and Calculation of weights
		% Adjust for masked out values by weighing other pixels in
		% same bin higher!

		correctionScaling = scaling;
		if INPUTVARS.doSolidAngleCorrection
			% SOLID ANGLE CORRECTION:
			SACorrection = MatFRAIA.corrections.SolidAngle(size(mask), correctionScaling, EXPERIMENT_INFO, INPUTVARS.doTiltCorrection);
			VAL = VAL.*SACorrection(keepIdx);
			% TODO: THE FOLLOWING NEEDS UPDATE!!!!!!:
% 			if nargout == 2
% 				infoOutput.corrections.SolidAngle = SACorrection;
% 			end
			clearvars SACorrection % Free up memory
		end

		if INPUTVARS.doPolarizationCorrection
			%SEE SUBFUNCTION FOR DESCRIPTION OF VARIABLES: INPUTVARS.PolarizationFactors = [PolCorrFactor, n]
			PolarizationCorrection = MatFRAIA.corrections.Polarization(TWOTHETA, AZI, INPUTVARS.PolarizationFactors(1), INPUTVARS.PolarizationFactors(2));
			VAL = VAL.*PolarizationCorrection;
			% TODO: THE FOLLOWING NEEDS UPDATE!!!!!!:
% 			if nargout == 2
% 				infoOutput.corrections.Polarization = PolarizationCorrection;
% 			end
			clearvars PolarizationCorrection % Free up memory
		end

		if INPUTVARS.doLorentzCorrection
			% LORENTZ CORRECTION (CORRECTING FOR A 2D AREA DETECTOR, WHEN USING DEBYE-SCHERRER GEOMETRY):
			%LorentzCorrection = corrections.Lorentz(size(mask), correctionScaling, EX, INPUTVARS.doTiltCorrection, INPUTVARS.LorentzCorrectionStartRadius, Radius, INPUTVARS.RadiusUnit);
			switch INPUTVARS.RadiusUnit
				case 'pixels'
				case 'degrees'
					R = [];
			end
			[LorentzCorrection, LorentzMean(SEGMENT,1:2)] = MatFRAIA.corrections.Lorentz(R, TWOTHETA, INPUTVARS.LorentzCorrectionStartRadius, Radius, INPUTVARS.RadiusUnit, scaling);
			VAL = VAL.*LorentzCorrection;
% 			if nargout >= 2
% 				infoOutput.corrections.Lorentz = LorentzCorrection;
% 			end
			clearvars LorentzCorrection % Free up memory
		end
		% Construct sparse matrix segment:
		if INPUTVARS.UseGPU
			INTIDX.TRANSMAT		= MatFRAIA.subfunctions.sparseInSteps(qBins,qPix,VAL./scaling^2,[Radius.N*Azi.N,numel(mask)],INPUTVARS.UseGPU);
			INTIDXCount.TRANSMAT	= MatFRAIA.subfunctions.sparseInSteps(qBins,qPix,  1./scaling^2,[Radius.N*Azi.N,numel(mask)],INPUTVARS.UseGPU);
		else
			INTIDX.TRANSMAT		= MatFRAIA.subfunctions.sparseInSteps(qPix,qBins,VAL./scaling^2,[numel(mask), Radius.N*Azi.N],INPUTVARS.UseGPU);
			INTIDXCount.TRANSMAT	= MatFRAIA.subfunctions.sparseInSteps(qPix,qBins,  1./scaling^2,[numel(mask), Radius.N*Azi.N],INPUTVARS.UseGPU);
		end
		
		if strcmpi(INPUTVARS.NormalizationMethod,'pixelCountInBin')
			pixCount = MatFRAIA.integrate(INTIDXCount,ones(numel(mask),1));
			% Adjust weights
			INTIDX.TRANSMAT = INTIDX.TRANSMAT./pixCount;
		end

	else
		INTIDX.TRANSMAT = [];%spalloc(Radius.N*Azi.N, numel(mask), 0);
		INTIDXCount.TRANSMAT = [];%INTIDX_segment(SEGMENT).TRANSMAT;
	end
	% Gather and prepare function output!
	if nargout >= 2
		infoOutput.EXPERIMENT_INFO = EXPERIMENT_INFO;
		infoOutput.INTEGRATION_OPTIONS	= INPUTVARS;
		if exist('BINCENTERS','var')
			infoOutput.BINCENTERS	= BINCENTERS;
			infoOutput.BINEDGES	= BINEDGES;
		else
			infoOutput.BINCENTERS	= [];
			infoOutput.BINEDGES	= [];
		end
	else
		infoOutput = struct;
	end

	if SEGMENT == 1
		INTIDX_ = INTIDX;
		INTIDXCount_ = INTIDXCount;
	else
		if ~isempty(INTIDX_.TRANSMAT) && ~isempty(INTIDX.TRANSMAT)
			INTIDX_.TRANSMAT = INTIDX_.TRANSMAT + INTIDX.TRANSMAT;
		elseif isempty(INTIDX_.TRANSMAT) && ~isempty(INTIDX.TRANSMAT)
			INTIDX_.TRANSMAT = INTIDX.TRANSMAT;
		end
		if ~isempty(INTIDXCount_.TRANSMAT) && ~isempty(INTIDXCount.TRANSMAT)
			INTIDXCount_.TRANSMAT = INTIDXCount_.TRANSMAT + INTIDXCount.TRANSMAT;
		elseif isempty(INTIDXCount_.TRANSMAT) && ~isempty(INTIDXCount.TRANSMAT)
			INTIDXCount_.TRANSMAT = INTIDXCount.TRANSMAT;
		end
	end
	if INPUTVARS.PrintInfo
		if SEGMENT > 1
			fprintf( repmat('\b',1,numel(printedMsg)) )
% 			fprintf('\n')
		else
			fprintf('------- RUN TIME --------');
			if INPUTVARS.UseLowMemoryFootprintIndexing
				fprintf(' (Low Memory-footprint indexing enabled) ')
			end
			fprintf('\n  ');
		end
		printedMsg = sprintf('  %d of %d segments completed after %s',SEGMENT,NOsegments,niceTime(starttime));
		fprintf('%s',printedMsg)
	end
end % Segment forloop end

if INPUTVARS.PrintInfo; fprintf('\n'); end
INTIDX = INTIDX_;
INTIDXCount = INTIDXCount_;
if INPUTVARS.UseGPU
	INTIDX.TRANSMAT = gather(INTIDX.TRANSMAT);
	INTIDXCount.TRANSMAT = gather(INTIDXCount.TRANSMAT);
else
	INTIDX.TRANSMAT = INTIDX.TRANSMAT.';
	INTIDXCount.TRANSMAT = INTIDXCount.TRANSMAT.';
end


if INPUTVARS.doMaskCorrection
	INTIDX = MatFRAIA.corrections.adjustMaskCorrection(INTIDX,INPUTVARS.MaskCorrectionThreshold,true(size(mask)),mask,INPUTVARS.UseGPU);
end

if INPUTVARS.doLorentzCorrection && INPUTVARS.LorentzCorrectionStartRadius
	LorentzMean(LorentzMean(:,1) == 0, 2) = 0;
	if ~any(LorentzMean == 0)
		INTIDX.TRANSMAT = INTIDX.TRANSMAT./(sum(LorentzMean(:,1).*LorentzMean(:,2))./(sum(LorentzMean(:,1))));
	end
	%IntegrationIndex.TRANSMAT = spfun(@(X) X/(sum(LorentzMean(:,1).*LorentzMean(:,2))./(sum(LorentzMean(:,1)))),IntegrationIndex.TRANSMAT);
end

if INPUTVARS.UseGPU
	if ~isa(INTIDX.TRANSMAT,'gpuArray')
		INTIDX.TRANSMAT = gpuArray(INTIDX.TRANSMAT);
	end
end
% if nargout == 3;varargout{1} = INTIDXCount;end
if INPUTVARS.doOutOfDetectorCorrection
	IntegrationIndexCorrection = MatFRAIA.corrections.OutOfDetectorCorrection(INTIDXCount,qREdges,INPUTVARS.RadiusUnit,Azi,samp2det,size(mask),INPUTVARS.UseGPU);
	INTIDX.TRANSMAT = INTIDX.TRANSMAT.*IntegrationIndexCorrection.TRANSMAT;
end

if INPUTVARS.PrintInfo
	fprintf('%s - Completed in %s: Indexing for unfolding\n',datestr(now,13),niceTime(starttime))
end
if INPUTVARS.Debugging
	echo(mfilename,'off')
end
end % End of main function


function smallestDataTypeFunctionHandle = reduceToIntDataType(ArrayContainingLargestNumber)
	largestNumber = max(reshape(ArrayContainingLargestNumber,[],1));
	%if isa(largestNumber,'float')
	
	if largestNumber <= maxint('uint8')
		smallestDataTypeFunctionHandle = @uint8;
	elseif largestNumber <= maxint('uint16')
		smallestDataTypeFunctionHandle = @uint16;
	elseif ~isa(largestNumber,'single') && largestNumber <= maxint('uint32')
		smallestDataTypeFunctionHandle = @uint32;
	else
		smallestDataTypeFunctionHandle = str2func(class(largestNumber));
	end
	
end
function EXPERIMENT_INFO = structFcn(CONVERTED,EXPERIMENT_INFO)
	if ~isfield(EXPERIMENT_INFO,'manuallySet')
		EXPERIMENT_INFO.manuallySet = {};
		EXPERIMENT_INFO.readFromPONI = {};
	end
	for fld = fieldnames(CONVERTED).'
		if ~isfield(EXPERIMENT_INFO,fld{1})
			disp('Was NOT field, but is now!')
			EXPERIMENT_INFO.readFromPONI{end+1} = fld{1};			
			EXPERIMENT_INFO.(fld{1}) = CONVERTED.(fld{1});
		else
			disp('Was ALREADY field')
			if isstruct(CONVERTED.(fld{1}))
				disp('Was struct')
			 	EXPERIMENT_INFO.(fld{1}) = structFcn(CONVERTED.(fld{1}),EXPERIMENT_INFO.(fld{1}));
			else
				disp('Was value (KEPT THE ONE SET MANUALLY)')
				EXPERIMENT_INFO.manuallySet{end + 1} = fld{1};
			end
			
			%EXPERIMENT_INFO.manuallySet.(fld{1}) = true;
		end
		
	end
end

