%% Load and integrate many files

clc
clearvars 
fclose all; 
startTime = tic;


% ========================= User variables =========================

%Overwrites existing integrated files if they already exist
overWriteIntegrals = false;

noWorkers = 6;			%Number of parrall workers
maxPicturesPerWorker = 25;	%Number of pictures any given worker can work on (increasing is ram expensive, but might be computionally smart)

if isempty(gcp('nocreate'))
	parpool(noWorkers);
end

%% Define folders
drive = 'C:'; %The drive used. should be changed based on OS

folders.experiment		= fullfile(drive,'Path','to','Experiment');
folders.data			= fullfile(folders.experiment,'path','to','data');
folders.integration	 	= fullfile(folders.experiment,'path','to','Integrated','data');
folders.saveFolder		= fullfile(folders.integration,'data');



%Ensure the savefolder exists
if ~isfolder(folders.saveFolder)
	mkdir(folders.saveFolder)
end

fprintf('%s - Fetching all h5 file names.\n',datestr(now,21))
allh5files = @() dir(fullfile(folders.data,'*data*.h5'));
allPaths2file = ""  + {allh5files.folder} + filesep + {allh5files.name};



%% ---------------------------------	Integration indexing		---------------------------------------------

fprintf('%s - Calculating integration index.\n',datestr(now,21))
poniSettings = {'pixsize',75e-6};
poniFile = 'lab6_scan_185.poni';

EX = MatFRAIA.subfunctions.PONI.readAndOrConvert(poniFile,poniSettings{:});

%% Integration settings
IntSettings.scaling	= 20; % The side of the square each pixel will be divided into. A higher number will produce more accurate data

% Azimuthal information
IntSettings.Azi.min	= 0; % Min azimuthal angle (in rad)
IntSettings.Azi.max	= 2*pi; % Max azimuthal angle (in rad)
IntSettings.Azi.N	  = 1;

% Radial information
IntSettings.Radius.min = 0; % Min radius (in degrees)
IntSettings.Radius.max = 30; % Max radius (in degrees)
IntSettings.Radius.N   = 1000; %numel(Radius.min:Radius.max); % Number of radial bins (2theta)
% If 1 pixel spacing is wanted then use: Radius.N = numel(Radius.min:Radius.max),
% if oversampeling is wanted multiply the before specified number with the
% wanted oversampeling

% MaskCorrectionThreshold is VERY mask (and Azi.N) specific, and thus needs
% to be tested for different experiments/masks/Azi.N's
% If the percentage of masked-out-pixels (in a bin) exceeds this value,
% the entire bin is set to nan, if not, it is corrected for
IntSettings.MaskCorrectionThreshold = 0.3;

IntSettings.RadiusUnit = 'degrees';

% Corrections
IntSettings.doLorentzCorrection = true;
IntSettings.doPolarizationCorrection = true;
IntSettings.doTiltCorrection = true;
IntSettings.doMaskCorrection =true;
IntSettings.doOutOfDetectorCorrection =true;
mask = ~logical(importdata(fullfile(folders.experiment,'path','to','mask.tif')).');


%% Indexing and integration

[IntegrationIndex, infoOut] = MatFRAIA.indexing(mask, EX, IntSettings);
IntegrationIndex_par = parallel.pool.Constant(IntegrationIndex);

% Integration
for h5fileIdx = numel(allh5files):-1:1
	% Generate full path to h5file containing images:
	path2file = allPaths2file{h5fileIdx};

	% Starting the integration of all files
	integrationQueue(h5fileIdx) = parfeval(@processPar,2,path2file,IntegrationIndex_par,folders,maxPicturesPerWorker,overWriteIntegrals);
end

%Waiting for the integration to complete, and reading the results
for idx = numel(integrationQueue):-1:1

	[completedIdx picsInFile(idx)] = fetchNext(integrationQueue);

	fprintf('%s (I = %d)\n',integrationQueue(completedIdx).Diary,idx)

	integrationQueue(completedIdx) = [];
end

fprintf('======== FINISHED IN (HH:MM:SS): %s==========\n',datestr(datenum(0,0,0,0,0,toc(startTime)),'HH:MM:SS'))

%% ----------------------------------------		Subfunctions		---------------------
function [picsInFile] = processPar(path2file,IntegrationIndex,folders,maxPicturesPerWorker,overWriteIntegrals)

	% First : check to see if the file has data
	infoFromPath = getInfoFromPath(path2file);
	try 
		FileInfo = h5info(path2file);
		picsInFile = FileInfo.Groups.Groups.Datasets.Dataspace.Size(3);
	catch ME
		fprintf('%s - FAILED LOADING: %s. ',datestr(now,21),infoFromPath.h5filename)
		fprintf('ERROR: %s: %s',ME.identifier,ME.message)
		picsInFile = 0;
		return
	
	end


	% Then prepare to integrate
	rounds 		= ceil(picsInFile/maxPicturesPerWorker);

	% integration loop
	for kk = rounds:-1:1
		% Figuring out what data to integrate
		readFrom = (kk-1)*maxPicturesPerWorker+1;
		if kk*maxPicturesPerWorker<picsInFile
			noPics	= maxPicturesPerWorker;
		else
			noPics	= picsInFile-readFrom+1;
		end
		% Loading the data for this round
		Ims 		= h5read(path2file,'/entry/data/data',[1 1 readFrom],[Inf Inf noPics]);
	
		% Processing the data for before integration
		Ims(1e9 < Ims) 	= 0;
		Ims 		= reshape(Ims,[],size(Ims,3));
		Ims 		= double(Ims);


		% Integrating the data
		IntegratedIm(:,:,readFrom:readFrom+noPics-1) 	= MatFRAIA.integrate(IntegrationIndex.Value,Ims);

	end

	fullSavePath = fullfile(folders.saveFolder,infoFromPath.h5filename);

	% Create the save file
	dataSize = size(IntegratedIm);
	if isfile(fullSavePath) && overWriteIntegrals 
		delete(fullSavePath)
	elseif isfile(fullSavePath)
		error(sprintf('The file %s already exists',fullSavePath))
	end

	h5create(fullSavePath, '/IntegratedData' ,dataSize,'ChunkSize',[dataSize(1) dataSize(2), 1], 'Deflate', 9);

	% Write the data to the file
	h5write(fullSavePath,'/IntegratedData',IntegratedIm);
end

function infoFromPath = getInfoFromPath(path2file)
	
	matchpattern = ['(?<=\' filesep, ')scan.*(?=\' filesep 'eiger)'];
	infoFromPath.scanNumber = regexp(path2file,matchpattern,'match','once','ignorecase');
	
	
	matchpattern = ['(?<=eiger.*\' filesep, ').*\.h5'];
	infoFromPath.h5filename = regexp(path2file,matchpattern,'match','once','ignorecase');
	
end
